Buffer = Buffer or {}
Buffer.BufferTypeUnknown = 0   -- unknown buffer type
Buffer.BufferTypeFile = 1      -- buffer contains a file's contents
Buffer.BufferTypeScratch = 2   -- scratch buffer
Buffer.BufferTypeDirEd = 3     -- directory editor buffer
Buffer.BufferTypeBufEd = 4     -- buffer list editor buffer
Buffer.BufferTypeGitEd = 5     -- git editor buffer
Buffer.BufferTypeFileTree = 6  -- FileTree
Buffer.BufferTypeSelection = 7 -- selection buffer

Buffer.type = function(type)
  if (type == Buffer.BufferTypeFile) then
    return "BufferTypeFile"
  end
  if (type == Buffer.BufferTypeScratch) then
    return "BufferTypeScratch"
  end
  if (type == Buffer.BufferTypeDirEd) then
    return "BufferTypeDirEd"
  end
  if (type == Buffer.BufferTypeBufEd) then
    return "BufferTypeBufEd"
  end
  if (type == Buffer.BufferTypeGitEd) then
    return "BufferTypeGitEd"
  end
  if (type == Buffer.BufferTypeFileTree) then
    return "BufferTypeFileTree"
  end
  if (type == Buffer.BufferTypeSelection) then
    return "BufferTypeSelection"
  end
  return "BufferTypeUnknown"
end

local function format_date(date)
  return os.date("%m/%d/%Y %H:%M:%S", os.time())
end

Buffer.dump = function(b)
  local buf = Buffer.ToLua(b);
  Console.Print("BUFFER\n")
  Console.Print("          name  " .. buf.name .. "\n")
  Console.Print("      fullpath  " .. buf.fullpath .. "\n")
  Console.Print("          type  " .. Buffer.type(buf.type) .. "\n")
  Console.Print("         flags  " .. math.tointeger(buf.flags) .. "\n")
  Console.Print("   line_number  " .. buf.line_number .. "\n")
  Console.Print("     num_lines  " .. buf.num_lines .. "\n")
  Console.Print("         ctime  " .. format_date(buf.ctime) .. "\n")
  Console.Print("         btime  " .. format_date(buf.mtime) .. "\n")
  Console.Print("LINES:\n")

  for i = 0, 20 do
    Console.Print(string.format("  x%3d %s\n", i + 1, Buffer.GetLine(b, i)));
  end
end

Buffer.active = function()
  Console.Clear();
  local win = Window.ActiveWindow()
  local buf = Window.GetBuffer(win)
  Buffer.dump(buf)
  Console.AnyKey();
end

Buffer.methods = function()
  Console.Clear()
  for k, v in pairs(Buffer) do
    Console.Print(tostring(k) .. " = " .. tostring(v) .. "\n")
  end
  Console.AnyKey()
end
